from zope.interface import Interface, Attribute

class ISalesforceBaseConnectorInfo(Interface):
    """Defines a READ ONLY interface for Salesforce
    """
    id = Attribute('id','Must be set to "portal_salesforcebaseconnector"')
        
    def describeGlobal():
        '''Retrieves list of available object Types
        (Account, Opportunity, etc., including custom
        types)'''
        
    def describeSObjects(sObjectTypes):
        '''Introspection on Salesforce datatypes. 
        Returns a complex data structure with properties for each Object 
        type passed in the List argument.'''
        
    def query(fields, sObjectType, conditionExpression=''):
        """Run a SoQL query against a Salesforce instance:
           portal.portal_salesforcebaseconnector.query(
                    ['FirstName',],'Contact',"FirstName='Stella'")
        """
        
    def queryMore(queryLocator):
        '''Query more record(s) in Salesforce'''
        
    def retrieve(fields, sObjectType, ids):
        '''Retrieve record(s) in Salesforce based on one SF Object ID's
        t.retrieve(['FirstName','Id'],'Contact','0033000000JKDFxAAP')
            or
        t.retrieve(['FirstName','Id'],'Contact',['0033000000JKDFxAAP'])         
        returns: [{'Id': '0033000000JKDFxAAP', 'FirstName': 'Siddartha'}]                
        '''
        
    def getDeleted(sObjectType, start, end):
        '''Get Salesforce Objects deleted between start and 
        end dates'''
        
    def getUpdated(sObjectType, start, end):
        '''Get Salesforce Objects updated between start and
        end dates'''
        
    def getUserInfo():
        '''Retrieves information about the authenticated user and
        returns it as a dictionary. This is the user that connects
        to Salesforce, not a Salesforce content object. This user
        and their password are set in the ZMI configuration form.'''

    def describeTabs():
        '''Describe the apps that have been configured for the user'''

    ## Not currently implemented in beatbox.
    #def describeLayout(sObjectType):
    #    '''Retrieves metadata about page layouts'''

    def setBatchSize(sObjectType):
        '''Set size of batch for queryMore()'''
        
class ISalesforceBaseConnector(ISalesforceBaseConnectorInfo):
    """Defines full interface for Salesforce
    """
    
    def setCredentials(username, password):
        '''Set username and password for SF. Returns Boolean'''
        
    def create(sObjects):
        '''Create record(s) in Salesforce
        sObjects are one or more dictionaries like this:
        obj = dict(type='Contact',
            LastName='Doe',
            FirstName='John',
            Phone='123-456-7890',
            Email='john@doe.com',
            Birthdate = datetime.date(1970, 1, 4)
        )'''
        
    def update(sObjects):
        '''Update record(s) in Salesforce
        data = dict(type='Contact',
            Id=id,
            Birthdate = newdate)
            '''

    def upsert(externalIdName, sObjects):
        '''If record(s) already exist in Salesforce an update is performed.
        Otherwise, new record(s) are created.'''

    def delete(ids):
        '''Delete record(s) in Salesforce by id or an array of ids.'''
